@@ -1,41 +1,60 @@
 package Finance::Shares::Sample;
-our $VERSION = 0.11;
+our $VERSION = 0.12;
 use strict;
 use warnings;
 use Exporter;
 use Carp;
 use Date::Calc qw(Today Date_to_Days Add_Delta_Days Delta_Days Day_of_Week);
 use Text::CSV_XS;
-use PostScript::Graph::Style 0.08;
-use Finance::Shares::MySQL   1.03;
+use PostScript::Graph::Style 1.00;
+use Finance::Shares::MySQL   1.04;
 
 our @ISA = qw(Exporter);
-our @EXPORT_OK = qw(%period %line 
-		    line_id function
+our @EXPORT_OK = qw(%period %function %functype
+		    line_id call_function
 		    today_as_string string_from_ymd ymd_from_string
 		    increment_days increment_date days_difference day_of_week);
 
 # Used for constructing Key text
 our %period = (
+    quotes   => 'quote',
+    weekdays => 'weekday',
     days     => 'day', 
     weeks    => 'week', 
     months   => 'month'
 );
 
 # Maps line function names to methods
-our %line = (
-    open_x   => sub { return 'open'; },
-    high_x   => sub { return 'high'; },
-    low_x    => sub { return 'low'; },
-    close_x  => sub { return 'close'; },
-    volume_x => sub { return 'volume'; },
-    value_p  => \&value,
-    value_v  => \&value,
+our %function = (
+    open   => sub { return 'open'; },
+    high   => sub { return 'high'; },
+    low    => sub { return 'low'; },
+    close  => sub { return 'close'; },
+    volume => sub { return 'volume'; },
+    value  => \&value,
+    price_value   => \&value,
+    volume_value  => \&value,
+    cycles_value  => \&value,
+    signals_value => \&value,
+);
+
+our %functype = (
+    open   => 'x',
+    high   => 'x',
+    low    => 'x',
+    close  => 'x',
+    volume => 'x',
+    value  => 'p',
+    price_value   => 'p',
+    volume_value  => 'v',
+    cycles_value  => 'c',
+    signals_value => 's',
 );
     
 our $highest_int = 10 ** 20;	# how do you set these properly?
 our $lowest_int = -$highest_int;
 our $points_margin = 0.02;	# logical 0/1 is mark_range -/+ points_margin * mark_range
+our $line_count = 10;		# keep track of order lines were added
 
 =head1 NAME
 
@@ -107,7 +126,7 @@ This array is a list of known dates indexed by the logical x coordinate.  It is
     $x    = $s->{lx}{$date};
     $date = $s->{dates}[$x];
     
-=head2 lines
+=head3 lines
 
 Function data is stored in this hash, first keyed by the chart where the function belongs - one of prices,
 volumes, cycles or signals.  Each of these are in turn sub-hashes keyed by a line id.  See <add_line> for details
@@ -130,6 +149,20 @@ It would be held within the Sample object thus.
     $s->{lines}{prices}{my_trend}{data}{2002-01-11} = 850;
                                        {2002-03-28} = 991;
 
+
+=head2 Functions
+
+A number of other modules provide functions which work on the data held here.  The 'functions' provided in this
+module are principally the source data, but they can be identified with the following text names when building
+model specifications.
+
+    open
+    close
+    high
+    low
+    volume
+    value
+
 =head1 CONSTRUCTOR
 
 =cut
@@ -143,13 +176,20 @@ sub new {
     bless( $o, $class );
     $o->{opt} = $opt;
     
-    $o->{lines}  = {};
+    $o->{lines}  = {
+	order   => [],
+	prices  => {},
+	volumes => {},
+	cycles  => {},
+	signals => {},
+	by_key  => {},   # lines indexed by <graph>::<key>
+    };
 
     ## option defaults
     $o->{stock}  = defined($opt->{symbol}) ? $opt->{symbol} : 'HPQ';
     $o->{end}    = defined($opt->{end_date})   ? $opt->{end_date}   : today_as_string();
     $o->{start}  = defined($opt->{start_date}) ? $opt->{start_date} : increment_date(today_as_string(), -1);
-    $o->{dtype}  = defined($opt->{dates_by})   ? $opt->{dates_by}   : 'days';
+    $o->{dtype}  = defined($opt->{dates_by})   ? $opt->{dates_by}   : 'quotes';
 
     ## fetch and process data
     carp "'source' must be specified\n" unless defined($opt->{source});
@@ -264,8 +304,33 @@ The four price values are typically decimals and the volume is usually an intege
 
 =head3 dates_by
 
-Control how the data are stored.  Suitable values are 'days', 'weeks' or 'months'.  For I<weeks> or I<months> the
-average price and volume data is given under the last known day.  (Default: 'days')
+Control how the data are stored and displayed on a chart.  Suitable values:
+
+=over 10
+
+=item quotes
+
+By default, the prices are displayed just as they are received.
+
+=item days
+
+Every day is recognized, though only trading days (Monday to Friday) will have quotes, and some of those may be
+missing.
+
+=item weekdays
+
+Entries are made for every day except weekends.  So the data mostly appears as for C<quotes> except that missing
+data is visible.
+
+=item weeks
+
+One entry shows the average data for each week.  Holes in the data are visible as blank weeks.
+
+=item months
+
+Entries show the average data for each month.
+
+=back
 
 =head3 end_date
 
@@ -280,7 +345,7 @@ exchange (e.g. BSY.L for BSkyB on the London Stock Exchange).
 =head3 mode
 
 Determines how the stock quotes are obtained if Finance::Shares::MySQL is used.  Suitable values are 'online',
-'offline' and 'cache'.  (Default: 'cache')
+'offline', 'fetch' and 'cache'.  (Default: 'cache')
 
 =head3 start_date
 
@@ -296,8 +361,30 @@ sub add_line {
     croak "No data for price line\nStopped" unless $data;
     croak "No key for price line\nStopped" unless $key;
     $show = 1 unless defined $show;
-    #warn "Sample: add_line($graph, $lineid, $data, $key, $style, $show)\n";
-    return $o->{lines}{$graph}{$lineid} = { data => $data, key => $key, style => $style, shown => $show };
+    my $min = $highest_int;
+    my $max = $lowest_int;
+    foreach my $value (values %$data) {
+	$min = $value if $value < $min;
+	$max = $value if $value > $max;
+    }
+    $o->{$graph}{min} = $min if $min < $highest_int and $min < $o->{$graph}{min};
+    $o->{$graph}{max} = $max if $max > $lowest_int and $max > $o->{$graph}{max};
+    
+    my $line = $o->{lines}{$graph}{$lineid} = {
+	data  => $data,
+	id    => $lineid,
+	order => $line_count++,
+	key   => $key,
+	style => $style,
+	shown => $show,
+	min   => $min,
+	max   => $max,
+    };
+    my $full_id = lc($graph . '::' . $key);
+    $o->{lines}{by_key}{$full_id} = $line;
+    push @{$o->{lines}{order}}, $line;
+
+    return $line;
 }
 
 =head2 add_line( graph, lineid, data, key [, style [, shown ]] )
@@ -337,6 +424,10 @@ as a hash with the following keys:
     shown   True if the line is to be drawn on a chart.
     style   A hash or PostScript::Graph::Style object.
     key	    A string associated with the style in the Key.
+    id      Unique internal identifier
+    order   Integer indicating when the line was added.
+    min	    The lowest data value
+    max	    The highest data value
 
 Example
 
@@ -353,10 +444,11 @@ sub value {
     my $o = shift;
     croak "No Finance::Shares::Sample object" unless ref($o) eq 'Finance::Shares::Sample';
     my %a = (
-	strict	=> 0,
-	shown	=> 1,
 	graph	=> 'prices',
 	value	=> 0,
+	strict	=> 0,
+	shown	=> 1,
+	style	=> undef,
 	@_);
 	
     my $id = line_id('value', $a{value});
@@ -457,26 +549,74 @@ Return a string indicating how the dates are spread.  One of 'data', 'days', 'wo
 
 =cut
 
-sub known_lines {
-    my ($o, @graphs) = @_;
-    @graphs = qw(prices volumes cycles signals) unless @graphs;
-    my @ids;
-    foreach my $graph (@graphs) {
-	foreach my $line (keys %{$o->{lines}{$graph}}) {
-	    push @ids, $line;
-	}
-    }
-    return wantarray ? @ids : \@ids;
+sub chart {
+    my ($o, $chart) = @_;
+    my $old = $o->{chart};
+    $o->{chart} = $chart if $chart;
+    return $old;
 }
 
-=head2 known_lines( [ graph(s) ] )
+=head2 chart( [chart] )
 
-Returns a list of line identifiers valid for the specified graphs, zero or more of prices, volumes, cycles or
-signals.  If none are specified, all known lines are returned.
+Either set or get the Finance::Shares::Chart displaying this data.
 
 =cut
 
-### SUPPORT METHODS
+sub lines {
+    my ($o, $array) = @_;
+    my $old = $o->{lines}{order};
+    $o->{lines}{order} = $array if ref($array) eq 'ARRAY';
+    return $old;
+}
+
+=head2 lines( [arrayref] )
+
+Set or get an array ref holding all the lines known to the sample so that the display order can be changed.
+They will be displayed by Finance::Shares::Chart in this order (as they were added) unless shuffling this array
+moves lines forward or backward.  See B<add_lines> for the structure of the line data.
+
+Note that this does not affect the key order, which will still show the order they were added.
+    
+Example 1
+
+    my $fss = new Finance::Shares::Sample(...);
+
+    # add several lines
+    $fss->add_line(...);
+    $fss->add_line(...);
+    ...
+
+    my $lines = $fss->lines();
+    @$lines = sort { -($a->{order} <=> $b->{order}) } @$lines
+    
+    my $fsc = new Finance::Shares::Chart(
+	sample => $fss,
+	...
+    );
+    $fsc->output(...);
+    
+The lines will be displayed on the chart in reverse order, showing those added first on top.
+
+Example 2
+
+    my $lines = $fss->lines();
+    my (@front, @back);
+    foreach my $line (@$lines) {
+	if ($line->{id} =~ /^signal/) { 
+	    push @front, $line;
+	} else {
+	    push @back, $line;
+	}
+    }
+    @$lines = (@back, @front);
+
+Here the signals are moved to the front.
+    
+=cut
+
+=head1 SUPPORT METHODS
+
+=cut
 
 sub fetch {
     my ($o, %opt) = @_;
@@ -536,7 +676,7 @@ sub prepare_dates {
     }
 
     ## process dates
-    my ($endday, $knowndate) = (0);
+    my ($endday, $enddate, $knowndate) = (0);
     my @prev = (0, 0, 0);
     my ($tdow, $tday, $tmonth, $tyear);
     my ($topen, $tclose, $thigh, $tlow, $tvolume, $total);
@@ -546,14 +686,48 @@ sub prepare_dates {
     my $pmax = $lowest_int;
     my $vmin = $highest_int;
     my $vmax = $lowest_int;
-    foreach my $date (sort keys %close) {
-	my @ymd     = ymd_from_string($date);
+    #foreach my $date (sort keys %close) {
+    #my @ymd     = ymd_from_string($date);
+    for (my $i = 0; $i <= $ndays; $i++) {
+    my @ymd     = increment_days(@first, $i);
+    my $date    = string_from_ymd(@ymd);
 	my ($year, $month, $day) = @ymd;
 	my $dow     = day_of_week(@ymd);
 	my $weekday = ($dow >= 1 and $dow <= 5);
 	my $known   = $close{$date};
 	
 	CASE: {
+	    ## quotes
+	    if ($dtype eq 'quotes') {
+		if ($known) {
+		    #print "$date, x=$x, i=$i\n";
+		    push @dates, $date;
+		    $lx{$date} = $x++;
+		    $pmin = $low{$date}  if $low{$date}  < $pmin;
+		    $pmax = $high{$date} if $high{$date} > $pmax;
+		    if ($volume{$date}) {
+			$vmin = $volume{$date} if $volume{$date} < $vmin;
+			$vmax = $volume{$date} if $volume{$date} > $vmax;
+		    }
+		}
+		last CASE;
+	    }
+
+	    ## weekdays
+	    if ($dtype eq 'weekdays') {
+		if ($weekday) {
+		    push @dates, $date;
+		    $lx{$date} = $x++;
+		    $pmin = $low{$date}  if defined($low{$date}) and $low{$date} < $pmin;
+		    $pmax = $high{$date} if defined($high{$date}) and $high{$date} > $pmax;
+		    if ($volume{$date}) {
+			$vmin = $volume{$date} if $volume{$date} < $vmin;
+			$vmax = $volume{$date} if $volume{$date} > $vmax;
+		    }
+		}
+		last CASE;
+	    }
+
 	    ## weeks
 	    if ($dtype eq 'weeks') {
 		# Each weeks data accumulates until the next week begins.
@@ -578,25 +752,29 @@ sub prepare_dates {
 			    delete $volume{$date};
 			    # note this as last date known so far
 			    $knowndate = $date;
+			} elsif ($dow == 5) {
+			    $knowndate = $date;
 			}
 			$tdow=$dow; $tday=$day; $tmonth=$month; $tyear=$year;
 		    } else {
 			# Monday
 			if (defined $knowndate) { 
 			    # put last weeks totals into last known date
-			    $open{$knowndate}   = $topen/$total;
-			    $high{$knowndate}   = $thigh/$total;
-			    $low{$knowndate}    = $tlow/$total;
-			    $close{$knowndate}  = $tclose/$total;
-			    $volume{$knowndate} = $tvolume/$total;
+			    if ($total) {
+				$open{$knowndate}   = $topen/$total;
+				$high{$knowndate}   = $thigh/$total;
+				$low{$knowndate}    = $tlow/$total;
+				$close{$knowndate}  = $tclose/$total;
+				$volume{$knowndate} = $tvolume/$total;
+				$pmin = $low{$knowndate}  if $low{$knowndate}  < $pmin;
+				$pmax = $high{$knowndate} if $high{$knowndate} > $pmax;
+				if ($volume{$knowndate}) {
+				    $vmin = $volume{$knowndate} if $volume{$knowndate} < $vmin;
+				    $vmax = $volume{$knowndate} if $volume{$knowndate} > $vmax;
+				}
+			    }
 			    push @dates, $knowndate;
 			    $lx{$knowndate} = $x++;
-			    $pmin = $low{$knowndate}  if $low{$knowndate}  < $pmin;
-			    $pmax = $high{$knowndate} if $high{$knowndate} > $pmax;
-			    if ($volume{$knowndate}) {
-				$vmin = $volume{$knowndate} if $volume{$knowndate} < $vmin;
-				$vmax = $volume{$knowndate} if $volume{$knowndate} > $vmax;
-			    }
 			}
 			# clear for a new week
 			if ($known) {
@@ -645,20 +823,25 @@ sub prepare_dates {
 		    } else {
 			# 1st working day of new month
 			if (defined $knowndate) { 
-			    $open{$knowndate}   = $topen/$total;
-			    $high{$knowndate}   = $thigh/$total;
-			    $low{$knowndate}    = $tlow/$total;
-			    $close{$knowndate}  = $tclose/$total;
-			    $volume{$knowndate} = $tvolume/$total;
-			    push @dates, $knowndate;
-			    $lx{$knowndate} = $x++;
-			    $pmin = $low{$knowndate}  if $low{$knowndate}  < $pmin;
-			    $pmax = $high{$knowndate} if $high{$knowndate} > $pmax;
-			    if ($volume{$knowndate}) {
-				$vmin = $volume{$knowndate} if $volume{$knowndate} < $vmin;
-				$vmax = $volume{$knowndate} if $volume{$knowndate} > $vmax;
+			    if ($total) {
+				$open{$knowndate}   = $topen/$total;
+				$high{$knowndate}   = $thigh/$total;
+				$low{$knowndate}    = $tlow/$total;
+				$close{$knowndate}  = $tclose/$total;
+				$volume{$knowndate} = $tvolume/$total;
+				$pmin = $low{$knowndate}  if $low{$knowndate}  < $pmin;
+				$pmax = $high{$knowndate} if $high{$knowndate} > $pmax;
+				if ($volume{$knowndate}) {
+				    $vmin = $volume{$knowndate} if $volume{$knowndate} < $vmin;
+				    $vmax = $volume{$knowndate} if $volume{$knowndate} > $vmax;
+				}
 			    }
+			} else {
+			    $knowndate = $enddate;
 			}
+			push @dates, $knowndate;
+			$lx{$knowndate} = $x++;
+			
 			# clear for a new month
 			if ($known) {
 			    $total   = 1;
@@ -677,15 +860,17 @@ sub prepare_dates {
 			$knowndate  = undef;
 		    }
 		    $endday = $day;
+		    $enddate = $date;
 		}
 		last CASE;
 	    }
 
 	    ## days - default CASE
 	    push @dates, $date;
-	    $lx{$date} = $x++;
-	    $pmin = $low{$date}  if $low{$date}  < $pmin;
-	    $pmax = $high{$date} if $high{$date} > $pmax;
+	    #$lx{$date} = $x++;
+	    $lx{$date} = $i;
+	    $pmin = $low{$date}  if defined($low{$date}) and $low{$date} < $pmin;
+	    $pmax = $high{$date} if defined($high{$date}) and $high{$date} > $pmax;
 	    if ($volume{$date}) {
 		$vmin = $volume{$date} if $volume{$date} < $vmin;
 		$vmax = $volume{$date} if $volume{$date} > $vmax;
@@ -720,22 +905,32 @@ sub prepare_dates {
     $o->{dates}  = \@dates;	# array where each index x => YYYY-MM-DD
     $o->{start}  = $dates[0];
     $o->{end}    = $dates[$#dates];
-    $o->{lines}{prices}{open}    = { shown => 0, key => 'Opening price', data => \%open };
-    $o->{lines}{prices}{high}    = { shown => 0, key => 'Highest price', data => \%high };
-    $o->{lines}{prices}{low}     = { shown => 0, key => 'Lowest price',  data => \%low };
-    $o->{lines}{prices}{close}   = { shown => 0, key => 'Closing price', data => \%close };
-    $o->{lines}{volumes}{volume} = { shown => 0, key => 'Volume',        data => \%volume } if %volume;
+    $o->{lines}{prices}{open} = {
+	shown => 0, order => 1, id => 'open', key => 'Opening price',
+	data => \%open, min => $pmin, max => $pmax };
+    $o->{lines}{prices}{high} = {
+	shown => 0, order => 2, id => 'high', key => 'Highest price',
+	data => \%high, min => $pmin, max => $pmax };
+    $o->{lines}{prices}{low} = {
+	shown => 0, order => 3, id => 'low', key => 'Lowest price',
+	data => \%low, min => $pmin, max => $pmax };
+    $o->{lines}{prices}{close} = {
+	shown => 0, order => 4, id => 'close', key => 'Closing price',
+	data => \%close, min => $pmin, max => $pmax };
+    $o->{lines}{volumes}{volume} = {
+	shown => 0, order => 5, id => 'volume', key => 'Volume',
+	data => \%volume, min => $vmin, max => $vmax } if %volume;
 
     ## Define min/max
     $o->{prices}{min} = $pmin;
     $o->{prices}{max} = $pmax;
     $o->{volumes}{min} = ($vmin != $highest_int) ? $vmin : 0;
     $o->{volumes}{max} = ($vmax != $lowest_int)  ? $vmax : 0;
-    my $margin = 100 * $points_margin;
-    $o->{cycles}{min} = -100 + $margin;
-    $o->{cycles}{max} = 100 - $margin;
-    $o->{signals}{min} = $margin;
-    $o->{signals}{max} = 100 - $margin;}
+    $o->{cycles}{min} = -1;
+    $o->{cycles}{max} = 1;
+    $o->{signals}{min} = 0;
+    $o->{signals}{max} = 1;
+}
 
 sub previous_date {
     my ($o, $date) = @_;
@@ -744,40 +939,67 @@ sub previous_date {
     return $prev || '';	
 }
 
-sub chart {
-    my ($o, $chart) = @_;
-    $o->{chart} = $chart if $chart;
-    return $o->{chart};
-}
-
-### SUPPORT FUNCTIONS
-
-sub line_id {
-    no warnings;
-    my $cmd = shift;
-    return $cmd unless @_;
-    my $args = join(',', @_);
-    return "$cmd($args)";
-}
-
-sub function {
-    my ($hash, $name, @args) = @_;
-    if (defined $name) {
-	my $fn = $hash->{$name};
-	no warnings;
-	#warn "Sample::function $name(", join(',', @args), ")\n";
-	if (defined $fn) {
-	    my $res;
-	    eval {
-		$res = &$fn(@args);
-	    };
-	    croak "error in $name()\n $@" if $@;
-	    return $res;
+sub known_lines {
+    my ($o, @graphs) = @_;
+    @graphs = qw(prices volumes cycles signals) unless @graphs;
+    my @ids;
+    foreach my $graph (@graphs) {
+	my @lines = values %{$o->{lines}{$graph}};
+	@lines = sort { $a->{order} <=> $b->{order} } @lines;
+	foreach my $entry (@lines) {
+	    push @ids, $entry->{id};
 	}
     }
-    return undef;	    
+    return wantarray ? @ids : \@ids;
 }
 
+=head2 known_lines( [ graph(s) ] )
+
+Returns a list of line identifiers valid for the specified graphs, zero or more of prices, volumes, cycles or
+signals.  If none are specified, all known lines are returned.
+
+=cut
+
+sub line_by_key {
+    my ($o, $full_id) = @_;
+    return $o->{lines}{by_key}{lc $full_id};
+}
+
+=head2 line_by_key( identifier )
+
+C<identifier> must be a concatention of the graph name, '::' and the line key (the visible one, NOT the line id),
+as given to B<add_line>.
+
+Returns the internal data structure for that line.  See L</add_line> for details.
+
+=cut
+
+sub min_value {
+    my ($o, $graph) = @_;
+    return $o->{$graph}{min};
+}
+
+=head2 min_value( graph )
+
+Return the lowest value used on the given graph, which should be one of prices, volumes, cycles or signals.
+
+=cut
+
+
+    
+sub max_value {
+    my ($o, $graph) = @_;
+    return $o->{$graph}{max};
+}
+
+=head2 max_value( graph )
+
+Return the highest value used on the given graph, which should be one of prices, volumes, cycles or signals.
+
+=cut
+
+
+    
 sub choose_line {
     my ($o, $graph, $id, $original) = @_;
 
@@ -831,14 +1053,14 @@ sub interpolate {
 	my ($px, $nx);
 	foreach my $x (0 .. $#{$o->{dates}}) {
 	    my $date = $o->{dates}[$x];
-	    if ($gdata->{$date}) {
+	    if (defined $gdata->{$date}) {
 		$px = $x;
 		$res->{$date} = $gdata->{$date};
 	    } else {
 		undef $nx;
 		foreach my $x1 ($x .. $#{$o->{dates}}) {
 		    my $date1 = $o->{dates}[$x1];
-		    $nx = $x1, last if $gdata->{$date1};
+		    $nx = $x1, last if defined $gdata->{$date1};
 		}
 		if (defined $px and defined $nx) {
 		    my $dn = $nx - $x;
@@ -852,13 +1074,74 @@ sub interpolate {
     }
 	
     $o->{lines}{$graph}{"_$id"} = {
+	%$given,
+	id    => "_$id",
 	data  => $res,
-	key   => $given->{key},
-	style => $given->{style},
 	shown => 0,
     };
 }
 
+
+=head1 SUPPORT FUNCTIONS
+
+=cut
+
+sub line_id {
+    no warnings;
+    my $cmd = shift;
+    return $cmd unless @_;
+    my $args = join(',', @_);
+    return "$cmd($args)";
+}
+
+=head2 line_id( arg1, arg2, ... )
+
+Builds an identifier from the list of strings passed as arguments.
+
+Note that this is NOT a method.
+
+=cut
+
+sub call_function {
+    my ($hash, $name, @args) = @_;
+    if (defined $name) {
+	my $fn = $hash->{$name};
+	no warnings;
+	#warn "Sample::call_function $name(", join(',', @args), ")\n";
+	if (defined $fn) {
+	    my @res;
+	    eval {
+		@res = &$fn(@args);
+	    };
+	    croak "error in $name()\n $@" if $@;
+	    return wantarray ? @res : $res[0];
+	}
+    }
+    return undef;	    
+}
+
+=head2 call_function ( hashref, name, args... )
+
+Call a known function by name.  The C<hashref> must map names to coderefs like \%function here or
+Finance::Shares::Model's \%testfunc.  C<name> must be one of the names recognised by that hash.  C<args> is
+the argument list passed to the function.  If a method is being called, an object must be the first argument in
+the C<args> list.
+
+Example
+
+    use Finance::Shares::Sample
+		    qw(call_function %function);
+    use Finance::Shares::Averages;
+
+    my $fss = new Finance::Shares::Sample(...);
+
+    my $res = call_function( \%function, 'simple_a', 
+	$fss, period => 3, key => 'Simple average' );
+
+Note this is NOT a method.
+
+=cut
+
 =head1 DATE FUNCTIONS
 
 There are three types of dates here.  A 'days' value is the number of days from some arbitrary day zero.  A 'date'
